Emotional = LibStub("AceAddon-3.0"):NewAddon("Emotional", "AceConsole-3.0", "AceEvent-3.0")

local defaults = {
    char = {
		configured = false,
		enableInPvE = true,
		enableInRaid = true,
		enableInGroup = true,
		enableInPvP = true,
		enableInLFR = false,
		enableDebug = false,
		enableGuildOnly = true,
		emotes = {
				applaud = false,
				cackle = false,
				cheer = false,
				gloat = false,
				golfclap = false,
				guffaw = false,
				moo = false,
				mourn = false,
				rolleyes = false,
				sigh = false,
				sniff = false,
				train = false,
				violin = false,
		},
		available = {
				}
		}
	}

local options = {
    name = "Emotional",
    handler = Emotional,
    type = 'group',
    args = {
		version = {
			order = 0,
			type = "description",
			name = "v8.2.0",
			width = "half",
		},
		desc = {
			order = 1,
			type = "description",
			name = "Death Makes Me Emotional",
			width = "0.75",
		},
		space3 = {
			order = 3,
			type = "description",
			name = "",
			width = "full",
		},
		space4 = {
			order = 4,
			type = "description",
			name = "",
			width = "full",
		},
		emote = {
			order = 20,
			type = "multiselect",
			name = "Emotion (Randomly from selected)",
			desc = "List of Available Emotions",
			get = "IsEnableEmotes",
			set = "ToggleEnableEmotes",
			width = "full",
			values = {
				["applaud"] = "You applaud at <target>. Bravo!",
				["cackle"] = "You cackle maniacally at <target>.",
				["cheer"] = "You cheer at <target>.",
				["gloat"] = "You gloat over <target>'s misfortune.",
				["golfclap"] = "You clap for <target>, clearly unimpressed.",
				["guffaw"] = "You take one look at <target> and let out a guffaw!",
				["moo"] = "You moo at <target>. Mooooooooooo.",
				["mourn"] = "In quiet contemplation, you mourn the death of <target>.",
				["rolleyes"] = "You roll your eyes at <target>.",
				["sigh"] = "You sigh a <target>.",
				["sniff"] = "You sniff <target>.",
				["train"] = "Choo Choo!",
				["violin"] = "You play the world's smallest violin for <target>.",
			}
		},
		enableIn = {
			order = 13,
			type = "description",
			name = "Enable",
			width = "half",
		},
		enableInPvE = {
			order = 16,
			type = "toggle",
			name = "PvE",
			desc = "Enable/Disable in PvE",
			get = "IsEnableInPvE",
			set = "ToggleEnableInPvE",
			width = "half"
		},
		enableInPvP = {
			order = 15,
			type = "toggle",
			name = "PvP",
			desc = "Enable/Disable in PvP",
			get = "IsEnableInPvP",
			set = "ToggleEnableInPvP",
			width = "half"
		},
		customize = {
			order = 30,
			type = "description",
			name = "Customization",
			width = "full",
		},
		enableGuildOnly = {
			order = 31,
			type = "toggle",
			name = "Guild Members Only",
			desc = "Enable/Disable Guild Only",
			get = "IsEnableGuildOnly",
			set = "ToggleEnableGuildOnly",	
			width = "full"			
		},
		enableInLFR = {
			order = 32,
			type = "toggle",
			name = "LFR",
			desc = "Enable/Disable in LFR",
			get = "IsEnableInLFR",
			set = "ToggleEnableInLFR",	
			width = "half"
		},		
		enableInRaid = {
			order = 33,
			type = "toggle",
			name = "Raids",
			desc = "Enable/Disable in Non-LFR Raids",
			get = "IsEnableInRaid",
			set = "ToggleEnableInRaid",
			width = "half"
		},
		enableInGroup = {
			order = 34,
			type = "toggle",
			name = "Groups",
			desc = "Enable/Disable in Non-Raid Groups",
			get = "IsEnableInGroup",
			set = "ToggleEnableInGroup",
			width = "half"
		},
		space40 = {
			order = 40,
			type = "description",
			name = "",
			width = "full",
		},
		space50 = {
			order = 50,
			type = "description",
			name = "",
			width = "full",
		},
		test = {
			order = 60,
			type = "execute",
			name = "Test Emotional",
			func = "RunTest",
		},
    },
}

function Emotional:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("EmotionalDB", defaults)
    LibStub("AceConfig-3.0"):RegisterOptionsTable("Emotional", options)
	self.EmotionalOptFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Emotional", "Emotional")
end

function Emotional:DisplayStatus()
	print("|cff0078c1Emotional|r"," Version: |cff0078c1",options.args.version.name,"|r")
	print(options.args.desc.name)
	if self.db.char.enableInPvP then
		print("|cff58b058PvP|r")
	else
		print("|cfff05032PvP|r")
	end
	if self.db.char.enableInPvE then
		print("|cff58b058PvE|r")
	else
		print("|cfff05032PvE|r")
	end
	if self.db.char.enableInLFR then
		print("|cff58b058LFR|r")
	else
		print("|cfff05032LFR|r")
	end
	if self.db.char.enableInRaid then
		print("|cff58b058Raids|r")
	else
		print("|cfff05032Raids|r")
	end
	if self.db.char.enableInGroup then
		print("|cff58b058Groups|r")
	else
		print("|cfff05032Groups|r")
	end
	self:DisplayEmotes()
end

function Emotional:DisplayInfo()
	print("Welcome to |cff0078c1Emotional|r"," Version: |cff0078c1",options.args.version.name,"|r Loaded")
	print(options.args.desc.name)
	print("Type |cff0078c1/emo|r to access GUI menu")
end

function Emotional:DisplayMenu()
	print("|cff0078c1Emotional|r"," Version: |cff0078c1",options.args.version.name,"|r")
	print(options.args.desc.name)
	print("/emo |cff0078c1-- GUI Menu|r")
	print("/emo lfr |cff0078c1-- Enable/Disable in LFR|r")	
	print("/emo pvp |cff0078c1-- Enable/Disable in PvP|r")
	print("/emo pve |cff0078c1-- Enable/Disable in PvE|r")
	print("/emo raid |cff0078c1-- Enable/Disable in Raid|r")
	print("/emo help |cff0078c1-- Display Help Menu|r")
	print("/emo status |cff0078c1-- Display Status|r")
end

function Emotional:ChatCommand(input)
	if input == "debug" then
		if (self.db.char.enableDebug) then
			self.db.char.enableDebug = false
			print("Debug set to |cfff05032Disabled|r")
		else
			self.db.char.enableDebug = true
			print("Debug set to |cff58b058Enabled|r")
		end
	elseif input == "raid" then
		if (self.db.char.enableInRaid) then
			self.db.char.enableInRaid = false
			print("Raid set to |cfff05032Disabled|r")
		else
			self.db.char.enableInRaid = true
			print("Raid set to |cff58b058Enabled|r")
		end
	elseif input == "pve" then
		if (self.db.char.enableInPvE) then
			self.db.char.enableInPvE = false
			print("PvE set to |cfff05032Disabled|r")
		else
			self.db.char.enableInPvE = true
			print("PvE set to |cff58b058Enabled|r")
		end
	elseif input == "pvp" then
		if (self.db.char.enableInPvP) then
			self.db.char.enableInPvP = false
			print("PvP set to |cfff05032Disabled|r")
		else
			self.db.char.enableInPvP = true
			print("PvP set to |cff58b058Enabled|r")
		end
	elseif input == "lfr" then
		if (self.db.char.enableInLFR) then
			self.db.char.enableInLFR = false
			print("LFR set to |cfff05032Disabled|r")
		else
			self.db.char.enableInLFR = true
			print("LFR set to |cff58b058Enabled|r")
		end
	elseif input == "guild" then
		if (self.db.char.enableGuildOnly) then
			self.db.char.enableGuildOnly = false
			print("Guild Only set to |cfff05032Disabled|r")
		else
			self.db.char.enableGuildOnly = true
			print("Guild Only set to |cff58b058Enabled|r")
		end	
	elseif input == "test" then
			self:RunEmote(self:ChooseEmote(),"none")
	elseif input == "status" then
			self:DisplayStatus()
	elseif input == "help" then
		self:DisplayMenu()
	elseif input == "reset" then
		self:Reset()
	elseif input == "" then
		InterfaceOptionsFrame_OpenToCategory(self.EmotionalOptFrame)
		InterfaceOptionsFrame_OpenToCategory(self.EmotionalOptFrame)
	else		
		self:RunEmote("rude","none")
		print("Try /emo help ")
	end
end

function Emotional:Reset()
	print("Emotional Reset to Default")
	self.db.char.enableInPvE = true
	self.db.char.enableInRaid = true
	self.db.char.enableInPvP = true
	self.db.char.enableInLFR = false
	self.db.char.enableDebug = false
	self.db.char.configured = false
	self.db.char.enableGuildOnly = false
end

function Emotional:DisplayEmotes()
	local available
	available = ""
	for i=table.getn(self.db.char.available),1,-1 do
		available = self.db.char.available[i] .. " " .. available
	end
	if (available == "") then
		self:NoEmotionSelected()
	else
		print("Emotions:",available)
	end
end

function Emotional:NoEmotionSelected()
		if math.random (5) == 5 then
			self:RunEmote("rolleyes","none")
		end
		print("Emotional is |cfff05032NOT|r configured")
end

function Emotional:OnEnable()
	if (UnitRace("player") ~= "Tauren") then
		options.args.emote.values["moo"] = nil
	end
	if (UnitRace("player") ~= "Worgen") then
		options.args.emote.values["sniff"] = nil
	end
	self:DisplayInfo()
	self:RegisterChatCommand("emo", "ChatCommand")
	self:RegisterChatCommand("emotional", "ChatCommand")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED","Action")
end

function Emotional:OnDisable()
	self:UnregisterChatCommand("emo", "ChatCommand")
	self:UnregisterChatCommand("emotional", "ChatCommand")
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function Emotional:Action(...)
	if not self.db.char.configured then
		InterfaceOptionsFrame_OpenToCategory(self.EmotionalOptFrame)
		InterfaceOptionsFrame_OpenToCategory(self.EmotionalOptFrame)
		self.db.char.configured = true
	end
	if (self:InLFR() and self.db.char.enableInLFR) or
	(((self:InPvP() and self.db.char.enableInPvP) or (self:InPvE() and self.db.char.enableInPvE)) and 
	((self:InRaid() and self.db.char.enableInRaid) or not self:InRaid()) and
	((self:InGroup() and self.db.char.enableInGroup) or not self:InGroup()) and
	(self:InRaid() or self:InGroup())) then
		local timestamp, event, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags = CombatLogGetCurrentEventInfo();
		if (event == "UNIT_DIED") and (UnitIsPlayer(destName))then
			if ((self:IsSameGuild(destName) and self.db.char.enableGuildOnly) or not self.db.char.enableGuildOnly) then
				self:RunEmote(self:ChooseEmote(),destName)		
				if (self.db.char.enableDebug) then
					print ("Number in group ",GetNumGroupMembers())
					print ("PvP",self:InPvP(),self.db.char.enableInPvP,"PvE",self:InPvE(),self.db.char.enableInPvE)
					print ("Raid",self:InRaid(),self.db.char.enableInRaid,"LRF",self:InLFR(),self.db.char.enableInLFR,"Party",self:InGroup(),self.db.char.enableInGroup)
					print(destName,"has died.")
				end
			end
		end
	end
end

function Emotional:InLFR()
	local name, type, difficultyIndex, difficultyName, maxPlayers, dynamicDifficulty, isDynamic, instanceMapId, lfgID = GetInstanceInfo()
	if (difficultyIndex == 7 or difficultyIndex == 17) then
		return true
	else
		return false
	end
end

function Emotional:InRaid()
	if (GetNumGroupMembers() > 5) then
		return true
	else
		return false
	end
end

function Emotional:InGroup()
	if ((GetNumGroupMembers() < 6) and (GetNumGroupMembers() > 0)) then
		return true
	else
		return false
	end
end

function Emotional:InPvP()
	return UnitIsPVP("player")
end

function Emotional:InPvE()
	return not UnitIsPVP("player")
end

function Emotional:RunTest()
	self:RunEmote(self:ChooseEmote(),"none")
end

function Emotional:IsEnableEmotes(info, key)
	return self.db.char.emotes[key]
end

function Emotional:IsSameGuild(target)
	local targetGuild, arg1, arg2 = GetGuildInfo(target)
	local playerGuild, arg3, arg4 = GetGuildInfo("player")
	if (self.db.char.enableDebug) then
		print (targetGuild)			
	end
	if (targetGuild == playerGuild) then
		return true
	else
		return false
	end
end

function Emotional:ToggleEnableEmotes(info, key, newValue)
	if newValue then
		self:RunEmote(key,"none")
		table.insert(self.db.char.available, key)
	else
		for i=1,table.getn(self.db.char.available),1 do
			if self.db.char.available[i] == key then
				table.remove(self.db.char.available, i)
				break
			end
		end
	end
	self.db.char.emotes[key] = newValue
end

function Emotional:RunEmote(emotion, target)
	if (emotion == "") then
		self:NoEmotionSelected()
	else
		DoEmote(emotion,target)
	end
	if (self.db.char.enableDebug) then
		if target == "none" then
			print(emotion)
		else
			print(emotion," at ",target)
		end
	end
end

function Emotional:ChooseEmote()
	if next(self.db.char.available) == nil then
		return ""
	else
		return self.db.char.available[math.random(table.getn(self.db.char.available))]
	end
end

function Emotional:IsEnableInPvE(info)
	return self.db.char.enableInPvE
end

function Emotional:IsEnableInRaid(info)
	return self.db.char.enableInRaid
end

function Emotional:IsEnableInGroup(info)
	return self.db.char.enableInGroup
end

function Emotional:IsEnableInPvP(info)
	return self.db.char.enableInPvP
end

function Emotional:IsEnableInLFR(info)
	return self.db.char.enableInLFR
end

function Emotional:IsEnableGuildOnly(info)
	return self.db.char.enableGuildOnly
end

function Emotional:ToggleEnableInPvE(info, newValue)
	self.db.char.enableInPvE = newValue
end

function Emotional:ToggleEnableInRaid(info, newValue)
	self.db.char.enableInRaid = newValue
end

function Emotional:ToggleEnableInGroup(info, newValue)
	self.db.char.enableInGroup = newValue
end

function Emotional:ToggleEnableInPvP(info, newValue)
	self.db.char.enableInPvP = newValue
end

function Emotional:ToggleEnableInLFR(info, newValue)
	self.db.char.enableInLFR = newValue
end

function Emotional:ToggleEnableGuildOnly(info, newValue)
	self.db.char.enableGuildOnly = newValue
end

function Emotional:IsEnableDebug(info)
	return self.db.char.enableDebug
end

function Emotional:ToggleEnableDebug(info, newValue)
	self.db.char.enableDebug = newValue
end